#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
# Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#        Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)
#              Joan Sala Calero (joan.sala@bsc.es)             
#

import sys
import logging
import datetime
import subprocess
import os
import configparser

def all_checks():
    return {
        "file_format":"test00_file_format",
        "standard_compliance":"test01_standard_compliance",
        "spatial_completeness":"test02_spatial_completeness",
        "temporal_completness":"test03_temporal_completeness",
        "spatial_consistency":"test04_spatial_consistency",
        "temporal_consistency":"test05_temporal_consistency",
        "valid_ranges":"test06_valid_ranges"
    }

def check_param_config(config, section, parameter):
    try:
        return config[section][parameter]         
    except:
        print('ERROR: Parameter {p} not defined. Check configuration file section [{s}]'.format(s=section, p=parameter))
        return None    

class checker_body:

    def __init__(self,name):
        self.name = name
        self.version = "0.0.1"
        self.help_mssg()

    def __version__(self):
        string = "\nData Quality Checker Wrapper Version: " + self.version + "\n"
        print(string)

    def help_mssg(self):
        string = "\n#"+ "#" * 80 + "\n#\n"
        string += "#"+ " " * 2+"Data Quality Checker Help message - How to run the DQC:\n#\n"
        string += "#"+ " " * 4+"$ python checker.py <config_file.ini>\n#\n"
        string += "#" + "#" * 80 + "\n"

        self.help = string


def main(args):
    dqc = checker_body("testing")
    dqc.__version__()

    # Help display
    try:
        if len(args) > 1:
            raise Exception('Too many arguments')
        if (args[0] in ["--help","-h","help","sos","commands"]):
            raise Exception('Help message')
        else:
            # Read configuration
            config = configparser.ConfigParser()
            config.read(args[0])
    except:
        print(dqc.help)
        sys.exit()        

    # Check mandatory fields in the general section    
    try:
        path_files = check_param_config(config, 'general', 'input')
        log_dir = check_param_config(config, 'general', 'log_dir')
        typef = check_param_config(config, 'general', 'type')
        # Not needed anymore
        config.remove_section('general')
    except:
        print('ERROR: Input directory not defined. Check configuration file section [general]')
        
    # For each section/test
    for s in config.sections(): 

        string = "\n#"+ "#" * 80 + "\n#\n"
        string += "#"+ " " * 2+"Data Quality Checker\n#\n"
        string += "#"+ " " * 4+"Check: {} \n#\n".format(s)
        string += "#" + "#" * 80 + "\n"
        print(string)        

        # Each section is a different test available
        try:
            if s == 'file_format':
                import test00_file_format
                file_format_check = test00_file_format.file_format_check(path_files)
                print(file_format_check.return_extensions_format())

            elif s == 'standard_compliance':            
                import test01_standard_compliance as test            
                standard_ck = test.standard_compliance_test(path_files, typef)
                standard_ck.main()

            elif s == 'spatial_completeness':            
                import test02_spatial_completeness as test
                spatial_completeness_ck = test.spatial_completeness_test(path_files, typef)
                spatial_completeness_ck.main()

            elif s == 'temporal_completeness':            
                import test03_temporal_completeness as test            
                # TODO: Implementation to be finished
                #temporal_completeness_ck.main()

            elif s == 'spatial_consistency':              
                grid_int = check_param_config(config, s, 'grid_interval')          
                
                with open(os.devnull, 'w') as fp:
                    subprocess.Popen(['utils/check_spatial_consistency.bash %s %s %s'%(typef, grid_int, path_files)], shell=True,stderr=fp)

                # TODO
                #import test04_spatial_consistency as test
                #spatial_consistency_ck = test.spatial_consistency_test(path_files, typef)
                #spatial_consistency_ck.main()
            
            elif s == 'temporal_consistency':
                time_int = check_param_config(config, s, 'time_interval')
                import test05_temporal_consistency as test
                #test.temporal_consistency_test(path_files, time_int)  
                #temporal_consistency_ck.main()          


            elif s == 'valid_ranges':
                import test06_valid_ranges as test
                valid_ranges_ck = test.valid_ranges_test(path_files, typef)
                #valid_ranges_ck.main()            


            else:
                raise Exception('Test {} is not available in the current checker implementation, please select one of these: {}'.format(s, all_checks().keys()))

        except Exception as e:
            print('ERROR: Processing section {}'.format(s))
            print('{}'.format(e))

if __name__ == "__main__":
    args = sys.argv[1:]

    sys.exit(main(args))
